import { Component, Prop, Event, h, State } from '@stencil/core';
import { dayClass, selectedDayToCalendarDay, dayNames, monthName, range, } from '../../utils/utils';
import { calendarMonth, getPreviousMonth, getNextMonth } from 'cntdys';
const chromeBordersFix = (table) => {
  table.style.borderSpacing = table.style.borderSpacing === '0px' ? '' : '0px';
};
/**
 * @slot back – Slot for the previous month button
 * @slot forward – Slot for the next month button
 */
export class CalyCalendar {
  constructor() {
    this.tables = [];
    /** (required) Year (YYYY) */
    this.year = new Date().getFullYear();
    /** (required) Month (1-12) */
    this.month = new Date().getMonth() + 1;
    /** (optional) Locale */
    this.locale = 'en-US';
    /** (optional) Start of the week. 0 for Sunday, 1 for Monday, etc. */
    this.startOfTheWeek = 0;
    /** (optional) Number of months rendered */
    this.numberOfMonths = 1;
    /** (optional) Show previous number of months */
    this.showPreviousNumberOfMonths = false;
    /** (optional) Range */
    this.range = false;
  }
  handleDayClick(day) {
    const dayInMonth = day.dayInMonth.toString().padStart(2, '0');
    const month = day.month.month.toString().padStart(2, '0');
    const selectedDay = `${day.month.year}-${month}-${dayInMonth}`;
    if (this.range) {
      if (!this.rangeStart) {
        this.rangeStart = selectedDay;
        this.rangeStartSelected.emit(selectedDay);
      }
      else if (!this.rangeEnd) {
        this.rangeEnd = selectedDay;
        this.rangeEndSelected.emit(selectedDay);
      }
      else {
        this.rangeStart = selectedDay;
        this.rangeEnd = null;
        this.rangeStartSelected.emit(selectedDay);
      }
    }
    else {
      this.selected = selectedDay;
      this.daySelected.emit(selectedDay);
    }
  }
  handleMouseOver(day) {
    if (this.range) {
      this.hoverDay = day;
      this.hoveredDay.emit(day);
    }
  }
  back() {
    const { month, year } = getPreviousMonth(this.year, this.month);
    this.month = month;
    this.year = year;
    this.tables.forEach(table => chromeBordersFix(table));
  }
  forward() {
    const { month, year } = getNextMonth(this.year, this.month);
    this.month = month;
    this.year = year;
    this.tables.forEach(table => chromeBordersFix(table));
  }
  render() {
    let month = { month: this.month, year: this.year };
    let months = [];
    for (let _i of range(this.numberOfMonths)) {
      let otherMonth = {
        year: month.year,
        month: month.month,
        weeks: calendarMonth(month.year, month.month, this.startOfTheWeek),
      };
      if (this.showPreviousNumberOfMonths) {
        months.unshift(otherMonth);
        month = getPreviousMonth(month.year, month.month); // going back
      }
      else {
        months.push(otherMonth);
        month = getNextMonth(month.year, month.month); // mutates month variable to progress it into next month
      }
    }
    return (h("div", { class: "grid" },
      h("section", { class: "navigation" },
        h("div", { onClick: () => this.back(), class: "button" },
          h("slot", { name: "back" }, "<")),
        h("div", { onClick: () => this.forward(), class: "button" },
          h("slot", { name: "forward" }, ">"))),
      months.map(month => (h("div", { class: "inline-flex" },
        h("div", { class: "flex justify-center" },
          monthName(month.year, month.month, this.locale),
          " ",
          month.year),
        h("table", { ref: el => (this.tables.includes(el) ? {} : this.tables.push(el)) },
          h("tr", null, dayNames(this.startOfTheWeek, this.locale).map(dayName => (h("td", { class: "borderless day-name" }, dayName)))),
          month.weeks.map(week => (h("tr", null, week.map(day => (h("td", { class: dayClass({
              weekDay: day,
              month: month.month,
              year: month.year,
              selectedDay: selectedDayToCalendarDay(this.selected),
              rangeStart: selectedDayToCalendarDay(this.rangeStart),
              rangeEnd: selectedDayToCalendarDay(this.rangeEnd),
              hoverDay: this.hoverDay,
              disableOnDay: this.disableOnDay,
            }), onClick: () => this.handleDayClick(day), onMouseOver: () => this.handleMouseOver(day) }, day.dayInMonth))))))))))));
  }
  static get is() { return "caly-calendar"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["caly-calendar.css"]
  }; }
  static get styleUrls() { return {
    "$": ["caly-calendar.css"]
  }; }
  static get properties() { return {
    "year": {
      "type": "number",
      "mutable": true,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "(required) Year (YYYY)"
      },
      "attribute": "year",
      "reflect": true,
      "defaultValue": "new Date().getFullYear()"
    },
    "month": {
      "type": "number",
      "mutable": true,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "(required) Month (1-12)"
      },
      "attribute": "month",
      "reflect": true,
      "defaultValue": "new Date().getMonth() + 1"
    },
    "selected": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "(optional) Selected day (dd-mm-yyyy)"
      },
      "attribute": "selected",
      "reflect": true
    },
    "locale": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "(optional) Locale"
      },
      "attribute": "locale",
      "reflect": false,
      "defaultValue": "'en-US'"
    },
    "startOfTheWeek": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "(optional) Start of the week. 0 for Sunday, 1 for Monday, etc."
      },
      "attribute": "start-of-the-week",
      "reflect": false,
      "defaultValue": "0"
    },
    "numberOfMonths": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "(optional) Number of months rendered"
      },
      "attribute": "number-of-months",
      "reflect": false,
      "defaultValue": "1"
    },
    "showPreviousNumberOfMonths": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "(optional) Show previous number of months"
      },
      "attribute": "show-previous-number-of-months",
      "reflect": false,
      "defaultValue": "false"
    },
    "disableOnDay": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "(timestamp: number) => boolean",
        "resolved": "(timestamp: number) => boolean",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "(optional) Disabled days"
      }
    },
    "range": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "(optional) Range"
      },
      "attribute": "range",
      "reflect": false,
      "defaultValue": "false"
    },
    "rangeStart": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "(optional) Range start (dd-mm-yyyy)"
      },
      "attribute": "range-start",
      "reflect": true
    },
    "rangeEnd": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "(optional) Range end (dd-mm-yyyy)"
      },
      "attribute": "range-end",
      "reflect": true
    }
  }; }
  static get states() { return {
    "hoverDay": {}
  }; }
  static get events() { return [{
      "method": "daySelected",
      "name": "daySelected",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "(optional) Event to listen for when new day is selected."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "rangeStartSelected",
      "name": "rangeStartSelected",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "(optional) Event to listen for when range start day is selected."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "rangeEndSelected",
      "name": "rangeEndSelected",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "(optional) Event to listen for when range end day is selected."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "hoveredDay",
      "name": "hoveredDay",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "(optional) Event to listen for what day is currently hovered."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
}
