import { attachShadow, createEvent, h, proxyCustomElement } from '@stencil/core/internal/client';
export { setAssetPath } from '@stencil/core/internal/client';

const isCurrentMonth = (day, year, month) => day.month.month === month && day.month.year === year;
const isWeekend = (day) => day.dayInWeek === 6 || day.dayInWeek === 0;
const isToday = (day, today) => day.dayInMonth === today.getDate() &&
  day.month.month === today.getMonth() + 1 &&
  day.month.year === today.getFullYear();
const isSelected = (weekDay, { day, year, month }) => day === weekDay.dayInMonth &&
  month === weekDay.month.month &&
  year === weekDay.month.year;
const selectedDayToCalendarDay = (selectedDay) => {
  if (!selectedDay)
    return;
  const [year, month, day] = selectedDay
    .split('-')
    .map(piece => parseInt(piece, 10));
  return { day, month, year };
};
const dayClass = ({ weekDay, month, year, selectedDay, rangeStart, rangeEnd, hoverDay, disableOnDay, }) => {
  const classes = ['day'];
  if (isWeekend(weekDay)) {
    classes.push('weekend');
  }
  if (isToday(weekDay, new Date())) {
    classes.push('today');
  }
  classes.push(isCurrentMonth(weekDay, year, month) ? 'current-month' : 'other-month');
  if (selectedDay && isSelected(weekDay, selectedDay)) {
    classes.push('selected');
  }
  const thisDayTs = Date.UTC(weekDay.month.year, weekDay.month.month - 1, weekDay.dayInMonth);
  if (rangeStart && (hoverDay || rangeEnd)) {
    const rangeStartTs = Date.UTC(rangeStart.year, rangeStart.month - 1, rangeStart.day);
    const hoverOrRangeEndTs = rangeEnd
      ? Date.UTC(rangeEnd.year, rangeEnd.month - 1, rangeEnd.day)
      : hoverDay
        ? Date.UTC(hoverDay.month.year, hoverDay.month.month - 1, hoverDay.dayInMonth)
        : undefined;
    if (rangeStartTs &&
      hoverOrRangeEndTs &&
      ((thisDayTs >= rangeStartTs && thisDayTs <= hoverOrRangeEndTs) ||
        (thisDayTs <= rangeStartTs && thisDayTs >= hoverOrRangeEndTs))) {
      classes.push('in-range');
    }
  }
  if (rangeStart && !rangeEnd) {
    classes.push('range-select-in-progress');
  }
  if (disableOnDay && disableOnDay(thisDayTs)) {
    classes.push('disabled');
  }
  return classes.join(' ');
};
const dayNames = (startOfTheWeek, locale = 'en-US') => {
  const days = [...Array(7).keys()].map(d => new Date(2017, 9, d + 1) // must not use UTC here
    .toLocaleString(locale, { weekday: 'long' })
    .slice(0, 2) // TODO: think of exposing this
  );
  for (let i = 6; i > 6 - startOfTheWeek; i--) {
    const day = days.shift();
    if (day) {
      days.push(day);
    }
  }
  return days;
};
const monthName = (year, month, locale = 'en-US') => new Date(year, month - 1).toLocaleString(locale, { month: 'long' }); // must not use UTC here
const range = n => Array.from(Array(n).keys());

const isLeap = (year) => year % 400 === 0 || (year % 100 !== 0 && year % 4 === 0);
const getDaysInMonth = (year, month) => {
    const daysInMonth = [31, -1, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
    if (month === 2 /* February */) {
        return isLeap(year) ? 29 : 28;
    }
    return daysInMonth[month - 1];
};
const getPreviousMonth = (year, month) => {
    if (month === 1 /* January */) {
        return {
            month: 12 /* December */,
            year: year - 1
        };
    }
    return {
        month: month - 1,
        year
    };
};
const getNextMonth = (year, month) => {
    if (month === 12 /* December */) {
        return {
            month: 1,
            year: year + 1
        };
    }
    return {
        month: month + 1,
        year
    };
};
const getPreviousDay = (year, month, day) => {
    if (day === 1) {
        const prevMonth = getPreviousMonth(year, month);
        const lastDayInPrevMonth = getDaysInMonth(prevMonth.year, prevMonth.month);
        const previousDayFromPreviousMonth = new Date(Date.UTC(prevMonth.year, prevMonth.month - 1, lastDayInPrevMonth));
        return {
            dayInMonth: previousDayFromPreviousMonth.getUTCDate(),
            dayInWeek: previousDayFromPreviousMonth.getUTCDay(),
            month: prevMonth
        };
    }
    const previousDay = new Date(Date.UTC(year, month - 1, day - 1));
    return {
        dayInMonth: previousDay.getUTCDate(),
        dayInWeek: previousDay.getUTCDay(),
        month: { month, year }
    };
};
const getNextDay = (year, month, day) => {
    const isLastDayInMonth = (y, m, d) => getDaysInMonth(y, m) === d;
    if (isLastDayInMonth(year, month, day)) {
        const nextMonth = getNextMonth(year, month);
        const nextDayOfNextMonth = new Date(Date.UTC(nextMonth.year, nextMonth.month - 1, 1));
        return {
            dayInMonth: nextDayOfNextMonth.getUTCDate(),
            dayInWeek: nextDayOfNextMonth.getUTCDay(),
            month: nextMonth
        };
    }
    const nextDay = new Date(Date.UTC(year, month - 1, day + 1));
    return {
        dayInMonth: nextDay.getUTCDate(),
        dayInWeek: nextDay.getUTCDay(),
        month: { month, year }
    };
};
/**
 * Get calendar for given month.
 *
 * @param {number} year - year [1900 – 2100]
 * @param {number} month - Month [1  - 12]
 * @param {number} [startOfTheWeek=0] - Start of the week [0 – 6] where 0 is Sunday, and 6 is Saturday
 *
 * @returns {Object[][]}
 */
function calendarMonth(year, month, startOfTheWeek = 0 /* Sunday */) {
    year = parseInt(year, 10);
    month = parseInt(month, 10);
    startOfTheWeek = parseInt(startOfTheWeek, 10);
    if (!year || isNaN(year) || year < 1900 || year > 2100) {
        throw Error('Year can be a number between 1900 and 3000');
    }
    if (!month || isNaN(month) || month < 1 || month > 12) {
        throw Error('Month can be a number from 1 and 12');
    }
    if ((!startOfTheWeek && startOfTheWeek !== 0) || isNaN(startOfTheWeek) || startOfTheWeek < 0 || startOfTheWeek > 6) {
        throw Error('Start of the week can be a number from 0 and 6');
    }
    const firstDayInMonth = new Date(Date.UTC(year, month - 1, 1));
    let startingDay = {
        dayInMonth: firstDayInMonth.getUTCDate(),
        dayInWeek: firstDayInMonth.getUTCDay(),
        month: { month, year }
    };
    // go back to first start of the week (Monday or Sunday, or any other really)
    if (startingDay.dayInWeek !== startOfTheWeek) {
        let previousDay = getPreviousDay(startingDay.month.year, startingDay.month.month, startingDay.dayInMonth);
        while (previousDay.dayInWeek !== startOfTheWeek) {
            previousDay = getPreviousDay(previousDay.month.year, previousDay.month.month, previousDay.dayInMonth);
        }
        startingDay = previousDay;
    }
    const DAYS_IN_WEEK = 7;
    const WEEKS_IN_CALENDAR = 6;
    const MONTH_CALENDAR_DAYS_NUMBER = DAYS_IN_WEEK * WEEKS_IN_CALENDAR;
    const days = [];
    let currentDay = startingDay;
    // then go next day up until all 42 (6 weeks) are filled
    while (days.length < MONTH_CALENDAR_DAYS_NUMBER) {
        days.push(currentDay);
        currentDay = getNextDay(currentDay.month.year, currentDay.month.month, currentDay.dayInMonth);
    }
    return [
        days.slice(DAYS_IN_WEEK * 0, DAYS_IN_WEEK * 1),
        days.slice(DAYS_IN_WEEK * 1, DAYS_IN_WEEK * 2),
        days.slice(DAYS_IN_WEEK * 2, DAYS_IN_WEEK * 3),
        days.slice(DAYS_IN_WEEK * 3, DAYS_IN_WEEK * 4),
        days.slice(DAYS_IN_WEEK * 4, DAYS_IN_WEEK * 5),
        days.slice(DAYS_IN_WEEK * 5, DAYS_IN_WEEK * 6)
    ];
}

const calyCalendarCss = ":host{font-family:var(--font, -apple-system, BlinkMacSystemFont);user-select:none}.grid{display:inline-grid;grid-template-areas:var(--grid,\n    \"nav nav\"\n    \"mn  mn\"\n  );column-gap:var(--grid-column-gap, 0px)}.disabled{pointer-events:none;color:var(--out-of-range-color, #d3d3d3)}.navigation{grid-area:nav;height:var(--navigation-height, 16px)}.inline-flex{display:inline-flex;flex-direction:column}.flex{display:flex}section{display:flex;justify-content:space-between}.justify-center{justify-content:center}.button{cursor:pointer}table{border-collapse:collapse}td{border-width:var(--cell-border-width, 1px);border-style:var(--cell-border-style, solid);border-color:var(--cell-border-color, #e4e7e7);width:var(--cell-width, 2.5rem);height:var(--cell-height, 2.5rem);text-align:var(--cell-text-align, center);font-size:var(--cell-font-size, 1rem)}td.borderless{border:none}td.day-name{font-size:var(--day-name-font-size, 1rem)}td.today{color:var(--today-color, #9e9c9c)}td.selected{background-color:var(--selected-bg-color, #00a699);color:var(--selected-color, white)}td.in-range{background-color:var(--in-range-bg-color, #00a699);color:var(--in-range-color, white)}td.other-month{visibility:var(--other-month-visibility, hidden);border:var(--other-month-border, none);}td.day:not(.other-month):hover{cursor:var(--day-cursor, pointer)}td.day:not(.other-month):not(.range-select-in-progress):hover{background-color:var(--hover-bg-color, gainsboro);color:var(--hover-color, black)}";

const chromeBordersFix = (table) => {
  table.style.borderSpacing = table.style.borderSpacing === '0px' ? '' : '0px';
};
const CalyCalendar = class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    attachShadow(this);
    this.daySelected = createEvent(this, "daySelected", 7);
    this.rangeStartSelected = createEvent(this, "rangeStartSelected", 7);
    this.rangeEndSelected = createEvent(this, "rangeEndSelected", 7);
    this.hoveredDay = createEvent(this, "hoveredDay", 7);
    this.tables = [];
    /** (required) Year (YYYY) */
    this.year = new Date().getFullYear();
    /** (required) Month (1-12) */
    this.month = new Date().getMonth() + 1;
    /** (optional) Locale */
    this.locale = 'en-US';
    /** (optional) Start of the week. 0 for Sunday, 1 for Monday, etc. */
    this.startOfTheWeek = 0;
    /** (optional) Number of months rendered */
    this.numberOfMonths = 1;
    /** (optional) Show previous number of months */
    this.showPreviousNumberOfMonths = false;
    /** (optional) Range */
    this.range = false;
  }
  handleDayClick(day) {
    const dayInMonth = day.dayInMonth.toString().padStart(2, '0');
    const month = day.month.month.toString().padStart(2, '0');
    const selectedDay = `${day.month.year}-${month}-${dayInMonth}`;
    if (this.range) {
      if (!this.rangeStart) {
        this.rangeStart = selectedDay;
        this.rangeStartSelected.emit(selectedDay);
      }
      else if (!this.rangeEnd) {
        this.rangeEnd = selectedDay;
        this.rangeEndSelected.emit(selectedDay);
      }
      else {
        this.rangeStart = selectedDay;
        this.rangeEnd = null;
        this.rangeStartSelected.emit(selectedDay);
      }
    }
    else {
      this.selected = selectedDay;
      this.daySelected.emit(selectedDay);
    }
  }
  handleMouseOver(day) {
    if (this.range) {
      this.hoverDay = day;
      this.hoveredDay.emit(day);
    }
  }
  back() {
    const { month, year } = getPreviousMonth(this.year, this.month);
    this.month = month;
    this.year = year;
    this.tables.forEach(table => chromeBordersFix(table));
  }
  forward() {
    const { month, year } = getNextMonth(this.year, this.month);
    this.month = month;
    this.year = year;
    this.tables.forEach(table => chromeBordersFix(table));
  }
  render() {
    let month = { month: this.month, year: this.year };
    let months = [];
    for (let _i of range(this.numberOfMonths)) {
      let otherMonth = {
        year: month.year,
        month: month.month,
        weeks: calendarMonth(month.year, month.month, this.startOfTheWeek),
      };
      if (this.showPreviousNumberOfMonths) {
        months.unshift(otherMonth);
        month = getPreviousMonth(month.year, month.month); // going back
      }
      else {
        months.push(otherMonth);
        month = getNextMonth(month.year, month.month); // mutates month variable to progress it into next month
      }
    }
    return (h("div", { class: "grid" }, h("section", { class: "navigation" }, h("div", { onClick: () => this.back(), class: "button" }, h("slot", { name: "back" }, "<")), h("div", { onClick: () => this.forward(), class: "button" }, h("slot", { name: "forward" }, ">"))), months.map(month => (h("div", { class: "inline-flex" }, h("div", { class: "flex justify-center" }, monthName(month.year, month.month, this.locale), " ", month.year), h("table", { ref: el => (this.tables.includes(el) ? {} : this.tables.push(el)) }, h("tr", null, dayNames(this.startOfTheWeek, this.locale).map(dayName => (h("td", { class: "borderless day-name" }, dayName)))), month.weeks.map(week => (h("tr", null, week.map(day => (h("td", { class: dayClass({
        weekDay: day,
        month: month.month,
        year: month.year,
        selectedDay: selectedDayToCalendarDay(this.selected),
        rangeStart: selectedDayToCalendarDay(this.rangeStart),
        rangeEnd: selectedDayToCalendarDay(this.rangeEnd),
        hoverDay: this.hoverDay,
        disableOnDay: this.disableOnDay,
      }), onClick: () => this.handleDayClick(day), onMouseOver: () => this.handleMouseOver(day) }, day.dayInMonth))))))))))));
  }
  static get style() { return calyCalendarCss; }
};

const CalyCalendar$1 = /*@__PURE__*/proxyCustomElement(CalyCalendar, [1,"caly-calendar",{"year":[1538],"month":[1538],"selected":[1537],"locale":[1],"startOfTheWeek":[2,"start-of-the-week"],"numberOfMonths":[2,"number-of-months"],"showPreviousNumberOfMonths":[4,"show-previous-number-of-months"],"disableOnDay":[16],"range":[4],"rangeStart":[1537,"range-start"],"rangeEnd":[1537,"range-end"],"hoverDay":[32]}]);
const defineCustomElements = (opts) => {
  if (typeof customElements !== 'undefined') {
    [
      CalyCalendar$1
    ].forEach(cmp => {
      if (!customElements.get(cmp.is)) {
        customElements.define(cmp.is, cmp, opts);
      }
    });
  }
};

export { CalyCalendar$1 as CalyCalendar, defineCustomElements };
